==> OpenCL-0.8.1.tgz: OpenCL-0.8.1/src/x917_rng.cpp <==
/*************************************************
* ANSI X9.17 RNG Source File                     *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/x917_rng.h>

namespace OpenCL {

/*************************************************
* Generate a random byte                         *
*************************************************/
byte X917_RNG::random()
   {
   if(position == MAX_OUT_PER_BUFFER)
      {
      generate(system_clock());
      if(++iteration == ITERATIONS_BEFORE_RESEED)
         reseed();
      }
   return buffer[position++];
   }

/*************************************************
* Generate a buffer of random bytes              *
*************************************************/
void X917_RNG::randomize(byte out[], u32bit length)
   {
   while(position && length) { out[0] ^= random(); out++; length--; }
   while(length >= MAX_OUT_PER_BUFFER)
      {
      generate(system_clock());
      if(++iteration == ITERATIONS_BEFORE_RESEED)
         reseed();
      xor_buf(out, buffer, MAX_OUT_PER_BUFFER);
      out += MAX_OUT_PER_BUFFER;
      length -= MAX_OUT_PER_BUFFER;
      }
   while(length) { out[0] ^= random(); out++; length--; }
   }

/*************************************************
* Refill the internal state                      *
*************************************************/
void X917_RNG::generate(u64bit input)
   {
   xor_buf(tstamp, (byte*)&input, 8);
   cipher->encrypt(tstamp);
   xor_buf(buffer, state, tstamp, cipher->BLOCK_SIZE);
   cipher->encrypt(buffer);
   xor_buf(state, buffer, tstamp, cipher->BLOCK_SIZE);
   cipher->encrypt(state);
   position = 0;
   }

/*************************************************
* Add entropy to internal state                  *
*************************************************/
void X917_RNG::add_randomness(const byte data[], u32bit length) throw()
   {
   while(length)
      {
      u32bit added = std::min(state.size(), length);
      xor_buf(state, data, added);
      generate(system_clock());
      length -= added;
      data += added;
      }
   reseed();
   }

/*************************************************
* Reseed the internal state                      *
*************************************************/
void X917_RNG::reseed()
   {
   SecureVector<byte> key(cipher->MAXIMUM_KEYLENGTH);
   generate(system_clock());
   state = buffer;
   generate(system_clock());
   tstamp = buffer;
   randomize(key, key.size());
   cipher->set_key(key, key.size());
   generate(system_time());
   iteration = 0;
   }

/*************************************************
* Clear memory of sensitive data                 *
*************************************************/
void X917_RNG::clear() throw()
   {
   cipher->clear();
   buffer.clear();
   tstamp.clear();
   state.clear();
   position = iteration = 0;
   }

/*************************************************
* X917 Constructor                               *
*************************************************/
X917_RNG::X917_RNG(BlockCipher* c, u32bit n) :
   RandomNumberGenerator("ANSI X9.17 RNG"),
   ITERATIONS_BEFORE_RESEED(n + 16),
   MAX_OUT_PER_BUFFER(c->BLOCK_SIZE / 2)
   {
   cipher = c;
   buffer.create(cipher->BLOCK_SIZE);
   state.create(cipher->BLOCK_SIZE);
   tstamp.create(cipher->BLOCK_SIZE);
   clear();
   reseed();
   }

}

==> OpenCL-0.8.3.tgz: OpenCL-0.8.3/src/x917_rng.cpp <==
/*************************************************
* ANSI X9.17 RNG Source File                     *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/x917_rng.h>

namespace OpenCL {

/*************************************************
* Generate a random byte                         *
*************************************************/
byte ANSI_X917_RNG::random()
   {
   if(position == MAX_OUT_PER_BUFFER)
      {
      generate(system_clock());
      if(++iteration == ITERATIONS_BEFORE_RESEED)
         reseed();
      }
   return buffer[position++];
   }

/*************************************************
* Generate a buffer of random bytes              *
*************************************************/
void ANSI_X917_RNG::randomize(byte out[], u32bit length)
   {
   while(position && length) { out[0] ^= random(); out++; length--; }
   while(length >= MAX_OUT_PER_BUFFER)
      {
      generate(system_clock());
      if(++iteration == ITERATIONS_BEFORE_RESEED)
         reseed();
      xor_buf(out, buffer, MAX_OUT_PER_BUFFER);
      out += MAX_OUT_PER_BUFFER;
      length -= MAX_OUT_PER_BUFFER;
      }
   while(length) { out[0] ^= random(); out++; length--; }
   }

/*************************************************
* Refill the internal state                      *
*************************************************/
void ANSI_X917_RNG::generate(u64bit input)
   {
   xor_buf(tstamp, (byte*)&input, 8);
   cipher->encrypt(tstamp);
   xor_buf(buffer, state, tstamp, cipher->BLOCK_SIZE);
   cipher->encrypt(buffer);
   xor_buf(state, buffer, tstamp, cipher->BLOCK_SIZE);
   cipher->encrypt(state);
   position = 0;
   }

/*************************************************
* Add entropy to internal state                  *
*************************************************/
void ANSI_X917_RNG::add_randomness(const byte data[], u32bit length) throw()
   {
   while(length)
      {
      u32bit added = std::min(state.size(), length);
      xor_buf(state, data, added);
      generate(system_clock());
      length -= added;
      data += added;
      }
   reseed();
   }

/*************************************************
* Reseed the internal state                      *
*************************************************/
void ANSI_X917_RNG::reseed()
   {
   SecureVector<byte> key(cipher->MAXIMUM_KEYLENGTH);
   generate(system_clock());
   state = buffer;
   generate(system_clock());
   tstamp = buffer;
   randomize(key, key.size());
   cipher->set_key(key, key.size());
   generate(system_time());
   iteration = 0;
   }

/*************************************************
* Clear memory of sensitive data                 *
*************************************************/
void ANSI_X917_RNG::clear() throw()
   {
   cipher->clear();
   buffer.clear();
   tstamp.clear();
   state.clear();
   position = iteration = 0;
   }

/*************************************************
* Return the name of this type                   *
*************************************************/
std::string ANSI_X917_RNG::name() const
   {
   return "ANSI X9.17<" + cipher->name() + "> RNG";
   }

/*************************************************
* X917 Constructor                               *
*************************************************/
ANSI_X917_RNG::ANSI_X917_RNG(BlockCipher* c, u32bit n) :
   ITERATIONS_BEFORE_RESEED(n + 16),
   MAX_OUT_PER_BUFFER(c->BLOCK_SIZE / 2)
   {
   cipher = c;
   buffer.create(cipher->BLOCK_SIZE);
   state.create(cipher->BLOCK_SIZE);
   tstamp.create(cipher->BLOCK_SIZE);
   clear();
   reseed();
   }

}
